package com.jusfoun.ydylsz.config;

import com.alibaba.druid.pool.DruidDataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.type.JdbcType;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.jdbc.DataSourceBuilder;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;

import javax.sql.DataSource;

@Configuration
@ConfigurationProperties(prefix = "spring.datasource2")
@MapperScan(basePackages = "com.jusfoun.ydylsz.secondaryMapper", sqlSessionFactoryRef = "db2SqlSessionFactory")
public class SecondaryDBConfig {
    @Value("${spring.datasource2.driverClassName}")
    private String driverClassName;
    @Value("${spring.datasource2.url}")
    private String url;
    @Value("${spring.datasource2.username}")
    private String username;
    @Value("${spring.datasource2.password}")
    private String password;
    @Value("${spring.datasource2.druid.initial-size}")
    private int initialSize;
    @Value("${spring.datasource2.druid.min-idle}")
    private int minIdle;
    @Value("${spring.datasource2.druid.max-active}")
    private int maxActive;
    @Value("${spring.datasource2.druid.max-wait}")
    private long maxWait;
    @Value("${spring.datasource2.druid.time-between-eviction-runs-millis}")
    private long timeBetweenEvictionRunsMillis;
    @Value("${spring.datasource2.druid.min-evictable-idle-time-millis}")
    private long minEvictableIdleTimeMillis;
    @Value("${spring.datasource2.druid.validation-query}")
    private String validationQuery;
    @Value("${spring.datasource2.druid.test-while-idle}")
    private boolean testWhileIdle;
    @Value("${spring.datasource2.druid.test-on-return}")
    private boolean testOnReturn;
    @Value("${spring.datasource2.druid.pool-prepared-statements}")
    private boolean poolPreparedStatements;
    @Value("${spring.datasource2.druid.max-pool-prepared-statement-per-connection-size}")
    private int maxPoolPreparedStatementPerConnectionSize;

    public DataSource getDb1DataSource(){
        return DataSourceBuilder.create().build();
    }

    @Bean(name = "db2DataSource")
    public DataSource primaryDataSource() {
        DruidDataSource dataSource = new DruidDataSource();

        // 配置基本属性
        dataSource.setDriverClassName(driverClassName);
        dataSource.setUsername(username);
        dataSource.setPassword(password);
        dataSource.setUrl(url);

        //初始化时建立物理连接的个数
        dataSource.setInitialSize(initialSize);
        //最大连接池数量
        dataSource.setMaxActive(maxActive);
        //最小连接池数量
        dataSource.setMinIdle(minIdle);
        //获取连接时最大等待时间，单位毫秒。
        dataSource.setMaxWait(maxWait);
        //一个连接在池中最小生存的时间
        dataSource.setMinEvictableIdleTimeMillis(minEvictableIdleTimeMillis);
        dataSource.setTestOnBorrow(true);
        dataSource.setTimeBetweenConnectErrorMillis(timeBetweenEvictionRunsMillis);
        dataSource.setValidationQuery(validationQuery);
        dataSource.setTestWhileIdle(testWhileIdle);
        dataSource.setTestOnReturn(testOnReturn);
        dataSource.setPoolPreparedStatements(poolPreparedStatements);
        dataSource.setMaxPoolPreparedStatementPerConnectionSize(maxPoolPreparedStatementPerConnectionSize);
        return dataSource;
    }

    @Bean("db2SqlSessionFactory")
    public SqlSessionFactory db1SqlSessionFactory(@Qualifier("db2DataSource") DataSource dataSource) throws Exception {
        SqlSessionFactoryBean bean = new SqlSessionFactoryBean();
        bean.setDataSource(dataSource);
        bean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath*:secondaryMapper/*.xml"));
        return bean.getObject();
    }

    @Bean("db2SqlSessionTemplate")
    public SqlSessionTemplate db1SqlSessionTemplate(@Qualifier("db2SqlSessionFactory") SqlSessionFactory sqlSessionFactory){
        return new SqlSessionTemplate(sqlSessionFactory);
    }
}
