package com.jusfoun.ydylsz.controller;

import com.github.pagehelper.PageInfo;
import com.jusfoun.ydylsz.service.*;
import com.jusfoun.ydylsz.service.zjbg.ZjbgService;
import com.jusfoun.ydylsz.vo.XwkTfzyb;
import com.jusfoun.ydylsz.vo.out.base.RootResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author by PIG
 * @date 2022/3/18 17
 */
@Api(description = "手动同步", tags = "手动同步")
@Log4j
@Controller
@RequestMapping(value = "/sync")
public class ManualSyncController {

    @Autowired
    private LsKcxxService lsKcxxService;

    @Autowired
    private LsrkService lsrkService;

    @Autowired
    private LsckService lsckService;

    @Autowired
    private LsrkjyService lsrkjyService;

    @Autowired
    private LsckjyService lsckjyService;

    @Autowired
    private ZjbgService zjbgService;


    @Autowired
    private HlhtCapturePicService capturePicService;

    @ApiOperation(value = "同步", notes = "参数：统一社会信用代码")
    @PostMapping(value = "/catch_pic")
    public RootResult<Object> catchPic() {
        log.info("======================= catch capture pic start ===================");
        long startTime = System.currentTimeMillis();
        capturePicService.catchPic();
        long overTime = System.currentTimeMillis();
        long l = overTime - startTime;
        log.info("======================= catch capture pic over ==================== " + l);
        RootResult<Object> result = new RootResult<>();
        Map<String, Object> resData = new HashMap<>();
        resData.put("time", l);
        result.setData(resData);
        return result;
    }

    @ApiOperation(value = "同步库存", notes = "参数：统一社会信用代码")
    @PostMapping(value = "/schedule_kc")
    public RootResult<Object> scheduleKc() {
        RootResult<Object> r = new RootResult<>();
        try {
            lsKcxxService.update();
            log.info("库存信息同步结束");
            r.setMsg("库存信息同步结束");
        } catch (Exception e) {
            e.printStackTrace();
            log.info("库存信息同步失败 {}" + e.getMessage());
            r.setMsg("库存信息同步失败");
        }
        return r;
    }

    @ApiOperation(value = "同步", notes = "参数：统一社会信用代码")
    @PostMapping(value = "/schedule")
    public RootResult<Object> page() {
        RootResult<Object> r = new RootResult<Object>();
        try {
            List<String> msgList = new ArrayList<>();
            //更新库存
            try {
                lsKcxxService.update();
                log.info("库存信息同步结束");
                msgList.add("库存信息同步结束");
            } catch (Exception e) {
                e.printStackTrace();
                msgList.add("库存信息同步失败");
                log.info("库存信息同步失败 {}" + e.getMessage());
            }
            //更新入库
            try {
                lsrkService.update();
                log.info("入库信息同步结束");
                msgList.add("入库信息同步结束");
            } catch (Exception e) {
                log.info("入库信息同步失败 {}" + e.getMessage());
                msgList.add("入库信息同步失败");
            }
            //更新出库
            try {
                lsckService.update();
                msgList.add("出库信息同步结束");
                log.info("出库信息同步结束");
            } catch (Exception e) {
                msgList.add("出库信息同步失败");
                log.info("出库信息同步失败 {}" + e.getMessage());
            }
            //更新入库检验
            try {
                lsrkjyService.update();
                log.info("入库检验信息同步结束");
                msgList.add("入库检验信息同步结束");
            } catch (Exception e) {
                msgList.add("入库检验信息同步失败");
                log.info("入库检验信息同步失败 {}" + e.getMessage());
            }
//		//更新出库质检
            try {
                lsckjyService.update();
                msgList.add("出库检验信息同步结束");
                log.info("出库检验信息同步结束");
            } catch (Exception e) {
                msgList.add("出库检验信息同步失败");
                log.info("出库检验信息同步失败 {}" + e.getMessage());
            }

            //更新质检报告
            try {
                zjbgService.synchronData();
                msgList.add("质检报告信息同步结束");
                log.info("质检报告信息同步结束");
            } catch (Exception e) {
                msgList.add("质检报告信息同步失败");
                log.info("质检报告信息同步失败 {}" + e.getMessage());
            }
            r.setData(msgList);
            r.setCode(200);
            r.setMsg("请求成功");
        } catch (Exception e) {
            r.setMsg(e.getMessage());
            e.printStackTrace();
            r.setCode(500);
        }
        return r;
    }

    @ApiOperation(value = "同步质检信息 入库检验 出库质检 质检报告", notes = "参数：统一社会信用代码")
    @PostMapping(value = "/schedule_zj")
    public RootResult<Object> scheduleZj() {
        RootResult<Object> r = new RootResult<Object>();
        try {
            List<String> msgList = new ArrayList<>();
            //更新入库检验
            try {
                lsrkjyService.update();
                log.info("入库检验信息同步结束");
                msgList.add("入库检验信息同步结束");
            } catch (Exception e) {
                msgList.add("入库检验信息同步失败");
                log.info("入库检验信息同步失败 {}" + e.getMessage());
            }
		    //更新出库质检
            try {
                lsckjyService.update();
                msgList.add("出库检验信息同步结束");
                log.info("出库检验信息同步结束");
            } catch (Exception e) {
                msgList.add("出库检验信息同步失败");
                log.info("出库检验信息同步失败 {}" + e.getMessage());
            }

            //更新质检报告
            try {
                zjbgService.synchronData();
                msgList.add("质检报告信息同步结束");
                log.info("质检报告信息同步结束");
            } catch (Exception e) {
                msgList.add("质检报告信息同步失败");
                log.info("质检报告信息同步失败 {}" + e.getMessage());
            }
            r.setData(msgList);
            r.setCode(200);
            r.setMsg("请求成功");
        } catch (Exception e) {
            r.setMsg(e.getMessage());
            e.printStackTrace();
            r.setCode(500);
        }
        return r;
    }

    @ApiOperation(value = "同步质检信息 质检报告", notes = "参数：统一社会信用代码")
    @PostMapping(value = "/schedule_quality_inspection_report")
    public RootResult<Object> scheduleQualityInspectionReport() {
        RootResult<Object> r = new RootResult<>();
        try {
            List<String> msgList = new ArrayList<>();
            //更新质检报告
            try {
                zjbgService.synchronData();
                msgList.add("质检报告信息同步结束");
                log.info("质检报告信息同步结束");
            } catch (Exception e) {
                msgList.add("质检报告信息同步失败");
                log.info("质检报告信息同步失败 {}" + e.getMessage());
            }
            r.setData(msgList);
            r.setCode(200);
            r.setMsg("请求成功");
        } catch (Exception e) {
            r.setMsg(e.getMessage());
            e.printStackTrace();
            r.setCode(500);
        }
        return r;
    }

    @ApiOperation(value = "同步入库信息 出库信息", notes = "参数：统一社会信用代码")
    @PutMapping(value = "/schedule_ckkc_rkkc")
    public RootResult<Object> scheduleCkkcRkkc() {
        RootResult<Object> r = new RootResult<>();
        try {
            List<String> msgList = new ArrayList<>();
            //更新入库
            try {
                lsrkService.update();
                log.info("入库信息同步结束");
                msgList.add("入库信息同步结束");
            } catch (Exception e) {
                log.info("入库信息同步失败 {}" + e.getMessage());
                msgList.add("入库信息同步失败");
            }
            //更新出库
            try {
                lsckService.update();
                msgList.add("出库信息同步结束");
                log.info("出库信息同步结束");
            } catch (Exception e) {
                msgList.add("出库信息同步失败");
                log.info("出库信息同步失败 {}" + e.getMessage());
            }
            r.setData(msgList);
            r.setCode(200);
            r.setMsg("请求成功");
        } catch (Exception e) {
            r.setMsg(e.getMessage());
            e.printStackTrace();
            r.setCode(500);
        }
        return r;
    }

}
