package com.jusfoun.ydylsz.controller.dxp;

import java.math.BigDecimal;
import java.util.LinkedHashMap;
import java.util.List;

import cn.hutool.core.net.url.UrlBuilder;
import com.alibaba.fastjson.JSONObject;
import com.jusfoun.ydylsz.utils.StringUtils;
import lombok.extern.log4j.Log4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.github.pagehelper.PageInfo;
import com.jusfoun.ydylsz.service.TfService;
import com.jusfoun.ydylsz.vo.XwkTfzyb;
import com.jusfoun.ydylsz.vo.out.base.RootResult;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

@Log4j
@RestController
@RequestMapping("/tfzy")
@Api(description="通风信息")
public class TfzyController {

	@Autowired
	private TfService tfService;

	//通风信息获取
    @ApiOperation(value = "查询全部通风信息", notes = "参数：统一社会信用代码")
    @GetMapping(value = "/list/{cfbh}")
    public RootResult<Object> onekey(@PathVariable("cfbh") String cfbh){
        RootResult<Object> r = new RootResult<Object>();
        try {
            List<XwkTfzyb> result = tfService.list(cfbh);
            r.setData(result);
            r.setCode(200);
            r.setMsg("请求成功");
        } catch (Exception e) {
            r.setMsg(e.getMessage());
            e.printStackTrace();
            r.setCode(500);
        }
        return r;
    }

    @ApiOperation(value = "通风详情", notes = "参数：统一社会信用代码")
    @GetMapping(value = "/info/{tfzyid}")
    public RootResult<Object> info(@PathVariable("tfzyid") String tfzyid){
        RootResult<Object> r = new RootResult<Object>();
        try {
            XwkTfzyb result = tfService.info(tfzyid);
            if(result!=null){
                String sl=result.getSl();
                if(!StringUtils.isEmpty(sl)){
                    BigDecimal b = new BigDecimal(sl);
                    BigDecimal divide = b.divide(BigDecimal.valueOf(1000d), 3);
                    String slStr = divide.toString();
                    result.setSlstr(slStr);
                }
            }
            r.setData(result);
            r.setCode(200);
            r.setMsg("请求成功");
        } catch (Exception e) {
            r.setMsg(e.getMessage());
            e.printStackTrace();
            r.setCode(500);
        }
        return r;
    }

    @ApiOperation(value = "通风分页", notes = "参数：统一社会信用代码")
    @PostMapping(value = "/page/{pageNum}/{pageSize}")
    public RootResult<Object> page(@RequestBody XwkTfzyb xwkTfzyb,@PathVariable("pageNum") Integer pageNum,@PathVariable("pageSize") Integer pageSize){
        RootResult<Object> r = new RootResult<Object>();
        try {
            PageInfo<XwkTfzyb> result = tfService.page(xwkTfzyb,pageNum,pageSize);
            r.setData(result);
            r.setCode(200);
            r.setMsg("请求成功");
        } catch (Exception e) {
            r.setMsg(e.getMessage());
            e.printStackTrace();
            r.setCode(500);
        }
        return r;
    }


    @ApiOperation(value = "通风详情 透传库级平台接口数据", notes = "通风记录id")
    @GetMapping(value = "/VentilationOperation/info/{tfzyid}")
    public RootResult<Object> getTfInfo(@PathVariable("tfzyid") String tfzyid){
        RootResult<Object> r = new RootResult<>();
        try {
            if (StringUtils.isNullString(tfzyid)) {
                r.setCode(500);
                r.setMsg("参数错误");
            } else {
                String URL = "http://10.223.3.62:9025/depot-qualitycheck/intelligents/VentilationOperationController/getById";
                UrlBuilder urlBuilder = UrlBuilder.of(URL, null);
                urlBuilder.addQuery("id", tfzyid);
                String requestUrl = urlBuilder.toString();
                System.out.println(requestUrl);
                RestTemplate restTemplate = new RestTemplate();
                LinkedMultiValueMap<String, Object> body = new LinkedMultiValueMap<>();
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
                // 请求体，包括请求数据 body 和 请求头 headers
                HttpEntity httpEntity = new HttpEntity(body, headers);
                ResponseEntity<JSONObject> responseEntity = null;
                try {
                    responseEntity = restTemplate.postForEntity(requestUrl,  httpEntity, JSONObject.class);
                    int statusCodeValue = responseEntity.getStatusCodeValue();
                    log.info("网络请求状态 = " + statusCodeValue);
                    if (200 == statusCodeValue) {
                        JSONObject resBody = responseEntity.getBody();
                        String retCode = resBody.getString("retCode");
                        log.info("数据查询状态 = " + retCode);
                        String message = resBody.getString("message");
    //                LinkedHashMap data = resBody.getObject("data", LinkedHashMap.class);
    //                JSONObject data = resBody.getJSONObject("data");
                        r.setData(resBody.getObject("data", LinkedHashMap.class));
                        r.setCode(200);
                        r.setMsg(retCode + " " + message);
                    } else {
                        log.error("------------网络请求错误-----------");
                        r.setCode(statusCodeValue);
                        r.setMsg("------------网络请求错误-----------");
                    }
                } catch (RestClientException e) {
                    e.printStackTrace();
                    r.setMsg(e.getMessage());
                    r.setCode(500);
                }
            }
        } catch (Exception e) {
            r.setMsg(e.getMessage());
            e.printStackTrace();
            r.setCode(500);
        }
        return r;
    }

}
