package com.jusfoun.ydylsz.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jusfoun.ydylsz.constants.RedisDataBaseIndex;
import com.jusfoun.ydylsz.constants.RedisKey;
import com.jusfoun.ydylsz.mapper.CommonMapper;
import com.jusfoun.ydylsz.mapper.UserInfoMapper;
import com.jusfoun.ydylsz.pojo.*;
import com.jusfoun.ydylsz.util.RedisUtil;
import com.jusfoun.ydylsz.vo.BarGrainPropertyVO;
import com.jusfoun.ydylsz.vo.BaseInfoSummarizeVO;
import com.jusfoun.ydylsz.vo.UserInfoVO;
import com.jusfoun.ydylsz.vo.XwkKcxx;
import com.unis.common.utils.CurrentUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import redis.clients.jedis.Jedis;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class CommonService {
	@Value("${roles.qlb}")
	private String qlb;
	@Value("${roles.gxgs}")
	private String gxgs;

    @Value("#{'${org.shi-liu-qu}'.split(',')}")
    private List<String> city6District;

	@Autowired
	private CommonMapper commonMapper;

	@Autowired
	SqlSessionTemplate sqlSessionTemplate;

	@Autowired
    UserInfoMapper userInfoMapper;

    public String getOrgAreaCode() {
        UserInfoVO userInfo = userInfoMapper.getByCode(CurrentUtil.getUserId());
        return userInfo.getOrgAreaCode();
    }

	public List<OrgResultVO> selectHwxxUnited(HwxxVO hwxxVO) {
		List<OrgInfoVO> orgList= commonMapper.selectOrgInfo(new OrgInfoVO());
		Map<String,OrgInfoVO > orgMap=orgList.stream().collect(  Collectors.toMap(OrgInfoVO::getOrgcode, a -> a));

		List<OrgResultVO> resList=new ArrayList<>();

		Map<String,Map<String,Map<String,HwxxVO>>> resultMap=new LinkedHashMap<>();
		List<HwxxVO> list= commonMapper.selectHwxxUnited(hwxxVO);
		for(HwxxVO temp : list){
			String orgCode=temp.getOrgcode();
			String parentCode=orgCode;
			if(orgCode.length()>8){
				parentCode=orgCode.substring(0,8);
			}
			Map<String,Map<String,HwxxVO>> map=null;
			boolean add=false;
			if(resultMap.containsKey(parentCode)){
				map=resultMap.get(parentCode);
			}else{
				map=new LinkedHashMap<>();
				resultMap.put(parentCode,map);
				add=true;
			}
			String lsxzbm=temp.getLsxzbm();
			if(StringUtils.isEmpty(lsxzbm)){
				lsxzbm="-";
			}
			if(map.containsKey(orgCode)){
				Map<String,HwxxVO> mmp=map.get(orgCode);
				if(mmp.containsKey(lsxzbm)){
					HwxxVO hwxx= mmp.get(lsxzbm);
					hwxx.setHwsl((hwxx.getHwsl()!=null ?hwxx.getHwsl() : 0)+(temp.getHwsl()!=null?temp.getHwsl():0));
				}else{
					mmp.put(lsxzbm, temp);
				}
			}else{
				Map<String,HwxxVO> mmp=new LinkedHashMap<>();
				mmp.put(lsxzbm, temp);
				map.put(orgCode,mmp);
				OrgResultVO orgResultVO=new OrgResultVO();
				orgResultVO.setMap(map);
				orgResultVO.setOrgCode(parentCode);
				if(orgMap.get(parentCode)!=null){
					String orgName=orgMap.get(parentCode).getOrgname();
					orgResultVO.setOrgName(orgName);
				}else{
					orgResultVO.setOrgName(parentCode);
				}
				if(add){
					resList.add(orgResultVO);
				}
			}
		}
		return resList;
	}

    public PageInfo<KcqkVO> selectKcqkPage(KcqkVO params, Integer pageNum, Integer pageSize) {
        String orgId = CurrentUtil.getOrgId();
        List<KcqkVO> kcxxList;

        Jedis jedis = RedisUtil.getJedis(RedisDataBaseIndex.DB2);
        String s = jedis.get(RedisKey.ORG_RELATE + orgId);
        JSONObject data = JSONObject.parseObject(s, JSONObject.class);
        RedisUtil.returnResource(jedis);
        if (data != null) {
            String type = data.getString("type");
            params.setType(type);
            JSONArray orgList = data.getJSONArray("orgList");
            params.setOrgList(orgList.toJavaList(String.class));
        }else {
            params.setType(null);
        }
        kcxxList = commonMapper.selectKcqkPage(params);
//            InventoryDetailVO inventoryDetailVO = commonMapper.getInventoryDetailVer2(Optional.ofNullable(kcTemp.getQybm()).orElse("-1"), kcTemp.getTyshxydm(), params.getQueryDate());
        List<InventoryDetailVO> inventoryDetailList = commonMapper.getInventoryDetailVer2(null, null, params.getQueryDate());
        Map<String, InventoryDetailVO> detailVOMap = inventoryDetailList.stream().collect(Collectors.toMap(InventoryDetailVO::getSskddm, Function.identity(), (key1, key2) -> key2));
        for (KcqkVO kcTemp : kcxxList) {
//            InventoryDetailVO inventoryDetail = detailVOMap.get(kcTemp.getTyshxydm());
            InventoryDetailVO inventoryDetail = commonMapper.getInventoryDetail(Optional.ofNullable(kcTemp.getQybm()).orElse("-1"), kcTemp.getTyshxydm());
//            if (null != inventoryDetail) {
//                InventoryTotalVO inventoryTotalVO = new InventoryTotalVO(inventoryDetail);
//                kcTemp.setInventoryDetail(inventoryDetail);
//                kcTemp.setInventoryTotal(inventoryTotalVO);
//            }
            if (null != inventoryDetail) {
                kcTemp.setInventoryDetail(inventoryDetail);
                InventoryTotalVO inventoryTotalVO = new InventoryTotalVO(inventoryDetail);
                kcTemp.setInventoryTotal(inventoryTotalVO);
            }
        }
        return new PageInfo<>(kcxxList, 10);
    }

	public PageInfo<WdVO> selectWdPage(WdVO wdVO, Integer pageNum ,Integer pageSize) {
		PageHelper.startPage(pageNum, pageSize);
		List<WdVO> kcxxList=null;
		if(qlb.equals(wdVO.getRoleId())){
            String areaCode = getOrgAreaCode();
            wdVO.setAreacode(areaCode);
			wdVO.setOrgcode(CurrentUtil.getOrgId());
			kcxxList = commonMapper.selectWdPageRole(wdVO);
		}else if(gxgs.equals(wdVO.getRoleId())){
			wdVO.setOrgcode(CurrentUtil.getOrgId());
			kcxxList = commonMapper.selectWdPageRole(wdVO);
		}else{
			kcxxList = commonMapper.selectWdPageLgd(wdVO);
		}
		//List<WdVO> kcxxList= commonMapper.selectWdPageLgd(wdVO);
		return new PageInfo<WdVO>(kcxxList);
	}

	public void test(){
		List<WdVO> list=new ArrayList<>();
		SqlSession sqlSession = sqlSessionTemplate.getSqlSessionFactory().openSession(ExecutorType.BATCH, false);
		CommonMapper mapper = sqlSession.getMapper(CommonMapper.class);
		for(WdVO wdVO : list){

		}
		sqlSession.commit();
	}

	public Object selectHwxxUnitedRole(HwxxVO hwxxVO) {
		List<HwVO> orgList= new ArrayList<>();
		//获取第一层级
		if(qlb.equals(hwxxVO.getRoleId())||gxgs.equals(hwxxVO.getRoleId())){
			HwVO hwVO = new HwVO();
			hwVO.setOrgcode(CurrentUtil.getOrgId());
			hwVO.setOrgname(CurrentUtil.getOrgName());
			hwVO.setTyshxydm(hwxxVO.getTyshxydm());
			hwVO.setHwVOList(new ArrayList<>());
			orgList.add(hwVO);
		}else{
			orgList = commonMapper.selectOrgFirst();
			for(HwVO hwVO:orgList){
				hwVO.setHwVOList(new ArrayList<>());
			}
		}
		//获取第二层级
		List<HwVO> resList=new ArrayList<>();

		List<HwVO> list=null;
		if(qlb.equals(hwxxVO.getRoleId())){
            String areaCode = getOrgAreaCode();
            list = commonMapper.selectHwxxUnitedRoles(areaCode, CurrentUtil.getOrgId());
			//10180008  orglevel

            //查询下属购销公司
            List<HwVO> gxlist = commonMapper.selectGxList(CurrentUtil.getOrgId());
            if (gxlist != null && gxlist.size() > 0) {
                HwVO hwVO = gxlist.get(0);
                HwVO hw = commonMapper.selectHwxxInfo(null, hwVO.getOrgcode());
                hwVO.setHwsl(hw.getHwsl());
                hwVO.setQthw(hw.getQthw() != null ? hw.getQthw() : 0);
                hwVO.setKcsl(hw.getKcsl());
                hwVO.setQjsl(hw.getQjsl() != null ? hw.getQjsl() : 0);
                hwVO.setSjsl(hw.getSjsl() != null ? hw.getSjsl() : 0);
                hwVO.setZyhw(hw.getZyhw() != null ? hw.getZyhw() : 0);
                List<HwVO> child = new ArrayList<>();
                for (HwVO hVO : list) {
                    if (hVO.getPcode().equals(hwVO.getOrgcode())) {
                        HwVO hws = commonMapper.selectHwxx(hVO.getOrgcode());
                        hVO.setHwsl(hws.getHwsl());
                        hVO.setQthw(hws.getQthw() != null ? hws.getQthw() : 0);
                        hVO.setKcsl(hws.getKcsl());
                        hVO.setQjsl(hws.getQjsl() != null ? hws.getQjsl() : 0);
                        hVO.setSjsl(hws.getSjsl() != null ? hws.getSjsl() : 0);
                        hVO.setZyhw(hws.getZyhw() != null ? hws.getZyhw() : 0);
                        child.add(hVO);
                    }
                }
                hwVO.setHwVOList(child);
                orgList.get(0).getHwVOList().add(hwVO);
            }
            for (HwVO hwVO : list) {
                if (orgList.get(0).getOrgcode().equals(hwVO.getPcode())) {
                    if (HwVO.ORG_LEVEL.GXGS.equalsVal(hwVO.getOrglevel())) {
                    } else {
                        HwVO hws = commonMapper.selectHwxx(hwVO.getOrgcode());
                        if (hws != null) {
                            hwVO.setHwsl(hws.getHwsl() != null ? hws.getHwsl() : 0);
                            hwVO.setQthw(hws.getQthw() != null ? hws.getQthw() : 0);
                            hwVO.setKcsl(hws.getKcsl());
                            hwVO.setQjsl(hws.getQjsl() != null ? hws.getQjsl() : 0);
                            hwVO.setSjsl(hws.getSjsl() != null ? hws.getSjsl() : 0);
                            hwVO.setZyhw(hws.getZyhw() != null ? hws.getZyhw() : 0);
                            orgList.get(0).getHwVOList().add(hwVO);
                        } else {
                            hwVO.setQthw(0);
                            hwVO.setQjsl(0d);
                            hwVO.setSjsl(0d);
                            hwVO.setZyhw(0);
                            orgList.get(0).getHwVOList().add(hwVO);
                        }

                    }
                }
            }
		}else if(gxgs.equals(hwxxVO.getRoleId())){
			list = commonMapper.selectHwxxUnitedRoles(null,CurrentUtil.getOrgId());
			for(HwVO hwVO:list){
				HwVO hw = commonMapper.selectHwxxInfo(null,CurrentUtil.getOrgId());
				hwVO.setHwsl(hw.getHwsl());
				hwVO.setQthw(hw.getQthw()!=null ?hw.getQthw() : 0);
				hwVO.setKcsl(hw.getKcsl());
				hwVO.setQjsl(hw.getQjsl()!=null ?hw.getQjsl() : 0);
				hwVO.setSjsl(hw.getSjsl()!=null ?hw.getSjsl() : 0);
				hwVO.setZyhw(hw.getZyhw()!=null ?hw.getZyhw() : 0);
			}
			orgList.get(0).setHwVOList(list);
		}else{
			list = commonMapper.selectHwxxUnits(hwxxVO);
			for(HwVO hwVO:orgList){
				for(HwVO hwsb:list){
					if(hwVO.getOrgcode().equals(hwsb.getPcode())){
						if(HwVO.ORG_LEVEL.GXGS.equalsVal(hwsb.getOrglevel())){
						}else{
							HwVO hws = commonMapper.selectHwxx(hwsb.getOrgcode());
							if(hws!=null){
								hwsb.setHwsl(hws.getHwsl()!=null ?hws.getHwsl() : 0);
								hwsb.setQthw(hws.getQthw()!=null ?hws.getQthw() : 0);
								hwsb.setKcsl(hws.getKcsl());
								hwsb.setQjsl(hws.getQjsl()!=null ?hws.getQjsl() : 0);
								hwsb.setSjsl(hws.getSjsl()!=null ?hws.getSjsl() : 0);
								hwsb.setZyhw(hws.getZyhw()!=null ?hws.getZyhw() : 0);
								hwVO.getHwVOList().add(hwsb);
							}
						}
					}
				}
				// 查询下属购销公司
				List<HwVO> gxlist = commonMapper.selectGxList(hwVO.getOrgcode());
				if(gxlist!=null&&gxlist.size()>0){
					HwVO hws = gxlist.get(0);
					HwVO hw = commonMapper.selectHwxxInfo(null,hws.getOrgcode());
                    if (null != hw) {
                        hws.setHwsl(hw.getHwsl()!=null ?hw.getHwsl() : 0);
                        hws.setQthw(hw.getQthw()!=null ?hw.getQthw() : 0);
                        hws.setKcsl(hw.getKcsl()!=null ?hw.getKcsl() : 0);
                        hws.setQjsl(hw.getQjsl()!=null ?hw.getQjsl() : 0);
                        hws.setSjsl(hw.getSjsl()!=null ?hw.getSjsl() : 0);
                        hws.setZyhw(hw.getZyhw()!=null ?hw.getZyhw() : 0);
                    }
					List<HwVO> child=new ArrayList<>();
					for(HwVO hVO:list){
						if(hVO.getPcode().equals(hws.getOrgcode())){
							HwVO hws1 = commonMapper.selectHwxx(hVO.getOrgcode());
							if(hws1!=null){
								hVO.setHwsl(hws1.getHwsl()!=null ?hws1.getHwsl() : 0);
								hVO.setQthw(hws1.getQthw()!=null ?hws1.getQthw() : 0);
								hVO.setKcsl(hws1.getKcsl()!=null ?hws1.getKcsl() : 0);
								hVO.setQjsl(hws1.getQjsl()!=null ?hws1.getQjsl() : 0);
								hVO.setSjsl(hws1.getSjsl()!=null ?hws1.getSjsl() : 0);
								hVO.setZyhw(hws1.getZyhw()!=null ?hws1.getZyhw() : 0);
								child.add(hVO);
							}
						}
					}
					hwVO.setHwVOList(child);
					//hwVO.getHwVOList().add(hwVO);
				}

			}

		}
		return orgList;
	}

    public Object listByRole(String qymc, String roleId) {
        String orgId = CurrentUtil.getOrgId();
        Map<String, Object> params = new HashMap<>();
        params.put("qymc", qymc);
        Jedis jedis = RedisUtil.getJedis(RedisDataBaseIndex.DB2);
        String s = jedis.get(RedisKey.ORG_RELATE + orgId);
        JSONObject data = JSONObject.parseObject(s, JSONObject.class);
        RedisUtil.returnResource(jedis);
        if (data != null) {
            String type = data.getString("type");
            params.put("type", type);
            JSONArray orgList = data.getJSONArray("orgList");
            params.put("orgList", orgList.toJavaList(String.class));
        }else {
            params.put("type", null);
        }

        List<BaseInfoSummarizeVO> list = commonMapper.listBaseInfo(params);
        list.forEach(BaseInfoSummarizeVO::initTotal);
        return list;
    }

    public KcqkVO getInventoryReport(String qybm) {
        XwkKcxx xwkKcxx = new XwkKcxx();
        xwkKcxx.setQybm(qybm);
        Map<String, KctjVO> lspzMap = new LinkedHashMap<>();
        Map<String, KctjVO> lsxzMap = new LinkedHashMap<>();
        Map<String, KctjVO> scnxMap = new LinkedHashMap<>();

        KcqkVO kcTemp = new KcqkVO();

        List<EchartsDouble> characterList = commonMapper.groupByCharacter(qybm);

        for (EchartsDouble item : characterList) {
            KctjVO entity = new KctjVO();
            entity.setKcsl(item.getValue().doubleValue());
            entity.setLsxzbmmc(item.getName());
            lsxzMap.put(item.getName(), entity);
        }
        List<XwkKcxx> list = commonMapper.groupByVarAndYear(qybm);

        LinkedHashMap<String, List<XwkKcxx>> nd = list.stream().collect(Collectors.groupingBy(XwkKcxx::getNd, LinkedHashMap::new, Collectors.toList()));
        nd.forEach((key, InventoryList) -> {
            Double kcsl = InventoryList.stream().map(XwkKcxx::getKcsl).reduce(0d, (curr, prod) -> BigDecimal.valueOf(curr).add(BigDecimal.valueOf(prod)).doubleValue());
            KctjVO entity = new KctjVO();
            entity.setKcsl(kcsl);
            entity.setNd(key);
            scnxMap.put(key, entity);
        });

        LinkedHashMap<String, List<XwkKcxx>> pz = list.stream().collect(Collectors.groupingBy(i -> i.getPzbmmc() + "@" + i.getPzbm(), LinkedHashMap::new, Collectors.toList()));
        pz.forEach((key, InventoryList) -> {
            String[] keys = key.split("@");
            Double kcsl = InventoryList.stream().map(XwkKcxx::getKcsl).reduce(0d, (curr, prod) -> BigDecimal.valueOf(curr).add(BigDecimal.valueOf(prod)).doubleValue());
            KctjVO entity = new KctjVO();
            entity.setKcsl(kcsl);
            entity.setPzbmmc(keys[0]);
            lspzMap.put(keys[1], entity);
        });

        Map.Entry<String, KctjVO> entry=null;
        KctjVO kctjVO=null;
        Iterator<Map.Entry<String, KctjVO>> iterator= lspzMap.entrySet().iterator();
        while(iterator.hasNext()){
            entry= iterator.next();
            kctjVO= entry.getValue();
            kctjVO.setKcslStr(String.format("%.2f",kctjVO.getKcsl()));
        }
        kcTemp.setLspzMap(lspzMap);

        iterator= lsxzMap.entrySet().iterator();
        while(iterator.hasNext()){
            entry= iterator.next();
            kctjVO= entry.getValue();
            kctjVO.setKcslStr(String.format("%.2f",kctjVO.getKcsl()));
        }
        kcTemp.setLsxzMap(lsxzMap);

        iterator= scnxMap.entrySet().iterator();
        while(iterator.hasNext()){
            entry= iterator.next();
            kctjVO= entry.getValue();
            kctjVO.setKcslStr(String.format("%.2f",kctjVO.getKcsl()));
        }
        kcTemp.setScnxMap(scnxMap);

        return kcTemp;
    }

    public JSONObject getInventoryReportBarData(String tyshxydm) {
        JSONObject object = new JSONObject();
        List<String> yearList = commonMapper.listYearByCode(tyshxydm);
        object.put("axisData", yearList);
        List<XwkKcxx> list = commonMapper.groupByVarAndYear(tyshxydm);
        LinkedHashMap<String, List<XwkKcxx>> collect = list.stream().collect(Collectors.groupingBy(i -> i.getPzbmmc() + "@" + i.getPzbm(), LinkedHashMap::new, Collectors.toList()));
        List<BarGrainPropertyVO> series = collect.entrySet().stream().map(item -> {
            String key = item.getKey();
            String[] keys = key.split("@");
            List<XwkKcxx> InventoryList = item.getValue();
            Map<String, Double> annualMap = InventoryList.stream().collect(Collectors.toMap(XwkKcxx::getNd, XwkKcxx::getKcsl));
            List<Double> seriesData = yearList.stream().map(annualMap::get).collect(Collectors.toList());
            return BarGrainPropertyVO.builder().type("bar").stack("total")
                    .name(keys[0])
                    .pzbm(keys[1])
                    .data(seriesData)
                    .build();
        }).collect(Collectors.toList());
        object.put("series", series);
        return object;
    }

    public JSONObject getInventoryProportion(String tyshxydm) {
        JSONObject object = new JSONObject();
        List<EchartsDouble> levelList = commonMapper.groupByLevel(tyshxydm);
        object.put("levelPie", levelList);
        List<EchartsDouble> characterList = commonMapper.groupByCharacter(tyshxydm);
        object.put("characterPie", characterList);
        return object;
    }

}
