package com.jusfoun.ydylsz.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jusfoun.ydylsz.constants.RedisDataBaseIndex;
import com.jusfoun.ydylsz.constants.RedisKey;
import com.jusfoun.ydylsz.mapper.DCfxxsjyDao;
import com.jusfoun.ydylsz.mapper.TzSlqyDao;
import com.jusfoun.ydylsz.mapper.UserInfoMapper;
import com.jusfoun.ydylsz.pojo.KqxxVO;
import com.jusfoun.ydylsz.util.RedisUtil;
import com.jusfoun.ydylsz.vo.*;
import com.unis.common.utils.CurrentUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import redis.clients.jedis.Jedis;

import java.util.*;

@Slf4j
@Service
public class CompanyService {

	@Value("${roles.qlb}")
	private String qlb;
	@Value("${roles.gxgs}")
	private String gxgs;

	@Value("#{'${org.shi-liu-qu}'.split(',')}")
    private List<String> city6District;
	@Autowired
	private TzSlqyDao tzSlqyDao;

	@Autowired
	private DCfxxsjyDao dCfxxsjyDao;

    @Autowired
    UserInfoMapper userInfoMapper;

    public String getOrgAreaCode() {
        UserInfoVO userInfo = userInfoMapper.getByCode(CurrentUtil.getUserId());
        return userInfo.getOrgAreaCode();
    }

	public List<CompanyVO> getCompanyList(String qymc) {
        Map<String, Object> params = new HashMap<>();
        params.put("qymc", qymc);
        String orgId = CurrentUtil.getOrgId();
        params.put("orgId", orgId);
        Jedis jedis = RedisUtil.getJedis(RedisDataBaseIndex.DB2);
        String s = jedis.get(RedisKey.ORG_RELATE + orgId);
        JSONObject data = JSONObject.parseObject(s, JSONObject.class);
        if (data != null) {
            String type = data.getString("type");
            params.put("type", type);
            JSONArray orgList = data.getJSONArray("orgList");
            params.put("orgList", Optional.ofNullable(orgList.toJavaList(String.class)).orElseGet(null));
        }else {
            params.put("type", null);
        }
		return tzSlqyDao.getCompanyList(params);
	}

	public PageInfo<CompanyVO> getCompanyList(CompanyQueryVO companyQueryVO, Integer pageNo, Integer pageSize) {
		PageHelper.startPage(pageNo, pageSize);
		List<CompanyVO> list = null;
//		if(qlb.equals(companyQueryVO.getRoleId())){
//            String areaCode = getOrgAreaCode();;
//            companyQueryVO.setAreacode(areaCode);
//			companyQueryVO.setOrgcode(CurrentUtil.getOrgId());
//			list = tzSlqyDao.getTypeRoleList(companyQueryVO);
//		}else if(gxgs.equals(companyQueryVO.getRoleId())){
//			companyQueryVO.setOrgcode(CurrentUtil.getOrgId());
//			list = tzSlqyDao.getTypeRoleList(companyQueryVO);
//		}else{
//			list = tzSlqyDao.getTypeList(companyQueryVO);
//		}

        Map<String, Object> params = new HashMap<>();
        String orgId = CurrentUtil.getOrgId();
        params.put("orgId", orgId);
        companyQueryVO.setOrgId(orgId);
        Jedis jedis = RedisUtil.getJedis(RedisDataBaseIndex.DB2);
        String s = jedis.get(RedisKey.ORG_RELATE + orgId);
        RedisUtil.returnResource(jedis);
        JSONObject data = JSONObject.parseObject(s, JSONObject.class);
        if (data != null) {
            String type = data.getString("type");
            params.put("type", type);
            companyQueryVO.setType(type);
            JSONArray orgList = data.getJSONArray("orgList");
            companyQueryVO.setOrgList(orgList.toJavaList(String.class));
        }else {
            params.put("type", null);
        }
//        list = tzSlqyDao.getTypeList(params);
        list = tzSlqyDao.getTypeList(companyQueryVO);

		return new PageInfo<>(list);
	}

	public PageInfo<CompanyVO> kucunList(CompanyVO cpyVO , Integer pageNo, Integer pageSize) {
		PageHelper.startPage(pageNo, pageSize);
		List<CompanyVO> dKdxxsjyVOs = tzSlqyDao.getkcList(cpyVO);
		//List<CompanyVO> dKdxxsjyVOs = tzSlqyDao.getkucunList(cpyVO);
		//Jedis jedis= RedisUtil.getJedis();
		//String result =jedis.get("kqList");
		//List<CompanyVO> ts = JSONArray.parseArray(result, CompanyVO.class);
		for(CompanyVO companyVO:dKdxxsjyVOs) {
			//查询库点库存
			String kc = tzSlqyDao.getZkc(companyVO.getTyshxydm());
			if(kc!=null){
				kc = String.format("%.5f",Double.valueOf(kc)/1000d).replaceAll("0+$", "");
				if(kc.endsWith(".")){
					kc = kc.substring(0,kc.length()-1);
				}
				companyVO.setKcsl(kc);
			}else{
				companyVO.setKcsl("0");
			}
			List<KucunObject> pzList = tzSlqyDao.getPzList(companyVO.getTyshxydm());
			List<KucunObject> xzList = tzSlqyDao.getXzList(companyVO.getTyshxydm());
			List<KucunObject> nxList = tzSlqyDao.getNxList(companyVO.getTyshxydm());
			String pzStr = "";
			String xzStr = "";
			String nxStr = "";
			for(KucunObject kucunObject:pzList){
				String val = String.format("%.5f",Double.valueOf(kucunObject.getValue())/1000d).replaceAll("0+$", "");
				if(val.endsWith(".")){
					val = val.substring(0,val.length()-1);
				}
				pzStr+=kucunObject.getName()+"("+val+") ";
			}
			for(KucunObject kucunObject:xzList){
				String val = String.format("%.5f",Double.valueOf(kucunObject.getValue())/1000d).replaceAll("0+$", "");
				if(val.endsWith(".")){
					val = val.substring(0,val.length()-1);
				}
				xzStr+=kucunObject.getName()+"("+val+") ";
			}
			for(KucunObject kucunObject:nxList){
				String val = String.format("%.5f",Double.valueOf(kucunObject.getValue())/1000d).replaceAll("0+$", "");
				if(val.endsWith(".")){
					val = val.substring(0,val.length()-1);
				}
				nxStr+=kucunObject.getName()+"("+val+") ";
			}
			companyVO.setPzStr(pzStr);
			companyVO.setXzStr(xzStr);
			companyVO.setNxStr(nxStr);
			//companyVO.setPzList(tzSlqyDao.getPzList(companyVO.getTyshxydm()));
			//companyVO.setXzList(tzSlqyDao.getXzList(companyVO.getTyshxydm()));
			//companyVO.setNxList(tzSlqyDao.getNxList(companyVO.getTyshxydm()));
		}
		return new PageInfo<CompanyVO>(dKdxxsjyVOs);
	}

	public List<ConpanyStyle> companystyle(String kqbm) {
		List<ConpanyStyle> list= tzSlqyDao.companystyle(kqbm);
		return list;
	}


    public List<CompanyVO> getCompanyRoleList(String qymc, String roleId) {
        List<CompanyVO> list = new ArrayList<>();
        Map<String, Object> params = new HashMap<>();
        params.put("qymc", qymc);
        String orgId = CurrentUtil.getOrgId();
        params.put("orgId", orgId);
        Jedis jedis = RedisUtil.getJedis(RedisDataBaseIndex.DB2);
        String s = jedis.get(RedisKey.ORG_RELATE + orgId);
        RedisUtil.returnResource(jedis);
        JSONObject data = JSONObject.parseObject(s, JSONObject.class);
        if (data != null) {
            String type = data.getString("type");
            params.put("type", type);
            JSONArray orgList = data.getJSONArray("orgList");
            params.put("orgList", orgList.toJavaList(String.class));
        }else {
            params.put("type", null);
        }
        list = tzSlqyDao.getCompanyList(params);
        return list;
    }

	public PageInfo<CompanyVO> kucunRoleList(CompanyVO cpyVO, Integer pageNo, Integer pageSize) {
		PageHelper.startPage(pageNo, pageSize);
        String orgId = CurrentUtil.getOrgId();
        Jedis jedis = RedisUtil.getJedis(RedisDataBaseIndex.DB2);
        String s = jedis.get(RedisKey.ORG_RELATE + orgId);
        RedisUtil.returnResource(jedis);
        JSONObject data = JSONObject.parseObject(s, JSONObject.class);
        if (data != null) {
            log.info("redis data ============= {}", data);
            String type = data.getString("type");
            cpyVO.setType(type);
            JSONArray orgList = data.getJSONArray("orgList");
            cpyVO.setOrgList(orgList.toJavaList(String.class));
        }
		List<CompanyVO> dKdxxsjyVOList = tzSlqyDao.getkcList(cpyVO);

		for(CompanyVO companyVO: dKdxxsjyVOList) {
			//查询库点库存
			String kc = tzSlqyDao.getZkc(companyVO.getTyshxydm());
			if(kc != null) {
				kc = String.format("%.5f",Double.valueOf(kc)/1000d).replaceAll("0+$", "");
				if(kc.endsWith(".")){
					kc = kc.substring(0,kc.length()-1);
				}
				companyVO.setKcsl(kc);
			}else {
				companyVO.setKcsl("0");
			}
			List<KucunObject> pzList = tzSlqyDao.getPzList(companyVO.getTyshxydm());
			List<KucunObject> xzList = tzSlqyDao.getXzList(companyVO.getTyshxydm());
			List<KucunObject> nxList = tzSlqyDao.getNxList(companyVO.getTyshxydm());
			String pzStr = "";
			String xzStr = "";
			String nxStr = "";
			for(KucunObject kucunObject:pzList){
				String val = String.format("%.5f",Double.valueOf(kucunObject.getValue())/1000d).replaceAll("0+$", "");
				if(val.endsWith(".")){
					val = val.substring(0,val.length()-1);
				}
				pzStr+=kucunObject.getName()+"("+val+") ";
			}
			for(KucunObject kucunObject:xzList){
				String val = String.format("%.5f",Double.valueOf(kucunObject.getValue())/1000d).replaceAll("0+$", "");
				if(val.endsWith(".")){
					val = val.substring(0,val.length()-1);
				}
				xzStr+=kucunObject.getName()+"("+val+") ";
			}
			for(KucunObject kucunObject:nxList){
				String val = String.format("%.5f",Double.valueOf(kucunObject.getValue())/1000d).replaceAll("0+$", "");
				if(val.endsWith(".")){
					val = val.substring(0,val.length()-1);
				}
				nxStr+=kucunObject.getName()+"("+val+") ";
			}
			companyVO.setPzStr(pzStr);
			companyVO.setXzStr(xzStr);
			companyVO.setNxStr(nxStr);
		}
		return new PageInfo<>(dKdxxsjyVOList);
	}

	public String selectByQybm(String qybm) {
		 KqxxVO kqxxVO= tzSlqyDao.selectByQybm(qybm);
		return  kqxxVO.getBrief_introduction();
	}
}
