package com.jusfoun.ydylsz.service;

import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jusfoun.ydylsz.pojo.InventoryQueryVO;
import com.jusfoun.ydylsz.pojo.KcxxVO;
import com.jusfoun.ydylsz.vo.*;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.time.DateFormatUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.jusfoun.ydylsz.mapper.DCfxxsjyDao;
import com.jusfoun.ydylsz.mapper.XwkKcxxMapper;

@Service
@Slf4j
public class LsKcxxService {

	@Autowired
	private XwkKcxxMapper wwkKcxxMapper;

	@Autowired
	private DCfxxsjyDao dCfxxsjyDao;

	public void update() {
		//查询时间内信息
		List<XwkKcxx> kcxxs = wwkKcxxMapper.selectKcxx(); // 原粮
		List<XwkKcxx> kcxxs2 = wwkKcxxMapper.selectCpKcxx(); // 成品粮+成品油
		//油罐库存同步
		List<XwkKcxx> crkyg = wwkKcxxMapper.selectCrkyg();//原油
		//List<XwkKcxx> kjyg = wwkKcxxMapper.selectKjyg();//成品油

		List<XwkKcxx> list=new ArrayList<>();
		list.addAll(kcxxs);
		list.addAll(kcxxs2);
		list.addAll(crkyg);
		//list.addAll(kjyg);

		List<XwkKcxx> errorKcxxList=new ArrayList<>();

		Map<String,List<XwkKcxx>> map=new HashMap<>();

		for(int i=0;i< list.size();i++) {
			XwkKcxx kcxx=list.get(i);
			String gb=kcxx.getGb();
			if(StringUtils.isNotEmpty(gb) && !gb.matches("\\d+")){
				errorKcxxList.add(kcxx);
			}
			String hwbm=kcxx.getHwbm();
			if(StringUtils.isNotEmpty(hwbm)) {//货位编码为null
				List<XwkKcxx> cList=map.get(hwbm);
				if(cList!=null){
					cList.add(kcxx);
				}else{
					cList=new ArrayList<>();
					cList.add(kcxx);
					map.put(hwbm, cList);
				}
			}
		}
		try {
			Thread.sleep(10000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		for(XwkKcxx xwkKcxx : errorKcxxList){
			log.error("数据gb为汉字,会导致数据或程序异常,请检查数据:"+ JSON.toJSONString(xwkKcxx));
		}

        Iterator<Map.Entry<String, List<XwkKcxx>>> it = map.entrySet().iterator();
        wwkKcxxMapper.deleteAll(); // 库级无法给空仓的数据，这边要全量删除
        while (it.hasNext()) {
            Map.Entry<String, List<XwkKcxx>> entry = it.next();
            String hwbm = entry.getKey();
            List<XwkKcxx> cList = entry.getValue();
//			wwkKcxxMapper.delete(hwbm);//删除
            if (cList != null) {
				XwkKcxxExample example;
                for (XwkKcxx kcxx : cList) {
					example = new XwkKcxxExample();
					XwkKcxxExample.Criteria c= example.createCriteria();
					c.andHwbmEqualTo(kcxx.getHwbm());
					/*List <XwkKcxx> list1 = wwkKcxxMapper.selectByExample(example);
					if(!list1.isEmpty()) continue;*/
                    log.info(JSON.toJSONString(kcxx));
                    kcxx.setCrtime(new Date());
                    String gb = kcxx.getGb();
                    if (StringUtils.isNotEmpty(gb) && !gb.matches("\\d+")) {
                        kcxx.setGb(null);
                    }
                    //kcxx.setLslb("0");
                    wwkKcxxMapper.insertSelective(kcxx);
                }
			}
		}
	}

	public List<KcxxVO> kcxxList(KcxxVO kcxxVO){
		List<KcxxVO> list= wwkKcxxMapper.kcxxList(kcxxVO);
		// 油罐信息
		DCfxxsjyVO vo = new DCfxxsjyVO();
		vo.setKddm(kcxxVO.getKddm());
		List<DCfxxsjyVO> ygList = dCfxxsjyDao.pageYgMxList(vo);
		ygList.forEach(dCfxxsjyVO -> {
			KcxxVO kcxx = new KcxxVO();
			kcxx.setHwbm(dCfxxsjyVO.getCfbh());
			kcxx.setPzbmmc(dCfxxsjyVO.getPzbm());
			kcxx.setLsxzbmmc(dCfxxsjyVO.getLsxzbm());
			kcxx.setCfmc(dCfxxsjyVO.getCfmc());
			kcxx.setKcsl(dCfxxsjyVO.getKcsl());
			kcxx.setPzbm(dCfxxsjyVO.getPzCode());
			kcxx.setLsxzbm(dCfxxsjyVO.getLsxzCode());
			list.add(kcxx);
		});
		for(KcxxVO kcxx: list){
			Double kcsl=kcxx.getKcsl();
			if(kcsl!=null){
                String kcslStr = String.format("%.3f", kcsl);
                kcxx.setKcslStr(kcslStr.replace(".000", ""));
			}
		}
		return list;
	}

    public PageInfo<InventoryQueryVO> listKcxx(CompositeQueryVo queryVo, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<InventoryQueryVO> list = dCfxxsjyDao.listInventoryQuery(queryVo);
        for (InventoryQueryVO item : list) {
            item.zeroToNull();
            item.numberFmt();
        }
        return PageInfo.of(list);
    }

    public InventoryQueryVO getTotal(CompositeQueryVo queryVo) {
        InventoryQueryVO totalInfo = dCfxxsjyDao.getInventoryQueryTotal(queryVo);
        if (null == totalInfo) {
            return new InventoryQueryVO();
        }
        totalInfo.numberFmt();
        return totalInfo;
    }

    public PageInfo<DCfxxsjyVO> listKcxxDetail(CompositeQueryVo queryVo, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<DCfxxsjyVO> list = dCfxxsjyDao.listInventoryDetail(queryVo);
        for (DCfxxsjyVO item : list) {
            item.numberFmt();
            Date rksj = item.getRksj();
			if(null != rksj) {
				String format = DateFormatUtils.format(rksj, "yyyy-MM");
				item.setRksjks(format);
			}
            item.setRksj(null);
        }
        return PageInfo.of(list);
    }

    public DCfxxsjyVO getDetailTotal(CompositeQueryVo queryVo) {
        DCfxxsjyVO totalInfo = dCfxxsjyDao.getInventoryQueryDetailTotal(queryVo);
        if (null == totalInfo) {
            return new DCfxxsjyVO();
        }
        totalInfo.numberFmt();
        return totalInfo;
    }

    public List<HouseVO> listDataByParams(JSONObject params) {
        String symbolSplit = "##";
        // 仓房
        List<WareHouseInventoryVO> list = dCfxxsjyDao.listWareHouseDataByParams(params);
        Map<String, List<WareHouseInventoryVO>> collect = list.stream()
                .collect(Collectors.groupingBy(e -> e.getHouseKey(symbolSplit), LinkedHashMap::new, Collectors.toList()));
        List<HouseVO> HouseInventoryList = collect.entrySet().stream().map(e -> {
            String key = e.getKey();
            String[] houseInfo = key.split(symbolSplit);
            List<WareHouseInventoryVO> value = e.getValue();
            return new HouseVO(houseInfo[0], houseInfo[1], value);
        }).collect(Collectors.toList());
        return HouseInventoryList;
    }
}
