package com.jusfoun.ydylsz.util;

import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by admin on 2018/7/23.
 */
public class DateTool {

	private static AtomicInteger counter = new AtomicInteger(0);
	private static Logger logger = LoggerFactory.getLogger(DateTool.class);
	private static Map<String, ThreadLocal<SimpleDateFormat>> sdfMap = new HashMap<String, ThreadLocal<SimpleDateFormat>>();

	static String yyyy_MM_dd="yyyy-MM-dd";//小写的mm表示的是分钟
	static String yyyy_MM_dd_HH_mm_ss="yyyy-MM-dd HH:mm:ss";//小写的mm表示的是分钟

	/**
	 * 返回一个ThreadLocal的sdf,每个线程只会new一次sdf
	 *
	 * @param pattern
	 * @return
	 */
	private static SimpleDateFormat getSimpleDateFormat(final String pattern) {
		ThreadLocal<SimpleDateFormat> tl = sdfMap.get(pattern);

		// 此处的双重判断和同步是为了防止sdfMap这个单例被多次put重复的sdf
		if (tl == null) {
			synchronized (DateTool.class) {
				tl = sdfMap.get(pattern);
				if (tl == null) {
					// 只有Map中还没有这个pattern的sdf才会生成新的sdf并放入map
					System.out.println("put new sdf of pattern " + pattern + " to map");
					// 这里是关键,使用ThreadLocal<SimpleDateFormat>替代原来直接new SimpleDateFormat
					tl = new ThreadLocal<SimpleDateFormat>() {
						@Override
						protected SimpleDateFormat initialValue() {
							System.out.println("thread: " + Thread.currentThread() + " init pattern: " + pattern);
							return new SimpleDateFormat(pattern);
						}
					};
					sdfMap.put(pattern, tl);
				}
			}
		}
		return tl.get();
	}


	/**
	 * 根据身份证号获取日期
	 * @author lipenghe
	 * @param IDCard
	 * @return Date
	 */
	public static Date getDateFromIDCard(String IDCard){
		if(IDCard==null){
			return null;
		}else{
			int len=IDCard.length();
			if(len==18){
				return DateTool.toDateTime(IDCard.substring(6,14), "yyyyMMdd");
			}else if(len==15){
				return DateTool.toDateTime("19"+IDCard.substring(6,12), "yyyyMMdd");
			}else{
				return null;
			}
		}
	}

	/**
	 * 判断2日期是否同一天
	 * @param date1
	 * @param date2
	 * @return
	 */
	public static boolean isSameDay(Date date1,Date date2){
		Calendar cal1 = Calendar.getInstance();
		cal1.setTime(date1);
		Calendar cal2 = Calendar.getInstance();
		cal2.setTime(date2);
		if((cal1.get(Calendar.YEAR)==cal2.get(Calendar.YEAR))&&(cal1.get(Calendar.MONTH)==cal2.get(Calendar.MONTH))
				&&(cal1.get(Calendar.DAY_OF_YEAR)==cal2.get(Calendar.DAY_OF_YEAR))){
			return true;
		}
		return false;
	}

	/**
	 * 将日期转换成中文日
	 * @author lipenghe
	 * @param date
	 * @return Date
	 */
	public static String toChinessYearMonth(Date date){
		if(date==null){
			return null;
		}else{
			Calendar calendar=Calendar.getInstance();
			calendar.setTime(date);
			Map<String,String> map=new HashMap<String,String>();
			map.put("0","Ο");
			map.put("1","一");
			map.put("2","二");
			map.put("3","三");
			map.put("4","四");
			map.put("5","五");
			map.put("6","六");
			map.put("7","七");
			map.put("8","八");
			map.put("9","九");
			StringBuilder sb=new StringBuilder();
			String c;
			String str=String.valueOf(calendar.get(Calendar.YEAR));
			for(int i=0,len=str.length();i<len;i++){
				c=String.valueOf(str.charAt(i));
				if(map.containsKey(c)){
					sb.append(map.get(c));
				}else{
					sb.append(c);
				}
			}
			sb.append("年");
			str=String.valueOf(calendar.get(Calendar.MONTH)+1);
			map.put("10","十");
			map.put("11","十一");
			map.put("12","十二");
			sb.append(map.get(str));
			sb.append("月");
			return sb.toString();
		}
	}

	/**
	 * 根据身份证号获取日期
	 * @author lipenghe
	 * @param IDCard
	 * @return Date
	 */
	public static String getDateStrFromIDCard(String IDCard){
		if(IDCard==null){
			return null;
		}else{
			int len=IDCard.length();
			if(len==18){
				return IDCard.substring(6,14);
			}else if(len==15){
				return "19"+IDCard.substring(6,12);
			}else{
				return "";
			}
		}
	}

	/**
	 * Object转日期
	 * @author lipenghe
	 * @param dateObj
	 * @return Date
	 */
	public static Date toDate(Object dateObj){
		if(dateObj!=null){
			if(dateObj instanceof Date){
				return (Date)dateObj;
			}else if(dateObj instanceof String){
				String str=dateObj.toString();
				if(str!=null&&str.length()>0){
					try {
						return getSimpleDateFormat(yyyy_MM_dd).parse(str);
					} catch (ParseException e) {
						e.printStackTrace();
					}
				}
			}
			return null;
		}else{
			return null;
		}
	}

	/**
	 * 字符串转换短日期类型:yyyy-MM-dd
	 * @author wangxuetao
	 * @param str
	 * @return Date
	 */
	public static Date toDate(String str){
		if(str!=null&&str.length()>0){
			try {
				return getSimpleDateFormat(yyyy_MM_dd).parse(str);
			} catch (ParseException e) {
				logger.error("日期字符串不合法",e);
			}
			return null;
		}
		return null;
	}

	/**
	 * 日期去除毫秒，以应对mybatis具体时间查询
	 * @author lipenghe
	 * @param  date
	 * @return Date
	 */
	public static Date getDateTimeWithoutMillis(Date date){
		if(date!=null){
			Calendar cal=Calendar.getInstance();
			cal.setTime(date);
			cal.set(Calendar.MILLISECOND,0);
			return cal.getTime();
		}
		return null;
	}

	/**
	 * 不带异常处理的可自定义转换格式的字符串转换日期类型
	 * @author lipenghe
	 * @param  str
	 * @param  format
	 * @return Date
	 */
	public static Date toDateTime(String str,String format){
		if(str!=null&&str.length()>0){
			SimpleDateFormat sdf=getSimpleDateFormat(format);
			try {
				return sdf.parse(str);
			} catch (ParseException e) {
				logger.error("日期字符串不合法",e);
			}
			return null;
		}
		return null;
	}

	/**
	 * 不带异常处理的可自定义转换格式的字符串转换日期类型
	 * @author lipenghe
	 * @param  str
	 * @return Date
	 */
	public static Date toDateTime(String str){
		if(str!=null&&str.length()>0){
			try {
				return getSimpleDateFormat(yyyy_MM_dd_HH_mm_ss).parse(str);
			} catch (ParseException e) {
				logger.error("日期字符串不合法",e);
			}
			return null;
		}
		return null;
	}

	/**
	 * 不带异常处理的Object转换日期类型:yyyy-MM-dd HH:mm:ss
	 * @author wangxuetao
	 * @param obj
	 * @return Date
	 */
	public static Date toDateTime(Object obj){
		if(obj !=null){
			if(obj instanceof Date){
				return (Date)obj;
			}else if(obj instanceof String){
				String str=obj.toString();
				if(str!=null&&str.length()>0){
					try {
						return getSimpleDateFormat(yyyy_MM_dd_HH_mm_ss).parse(obj.toString());
					} catch (ParseException e) {
						logger.error("日期字符串不合法",e);
					}
				}
			}
			return null;
		}
		return null;
	}

	/**
	 * 可自定义转换格式的字符串转换日期类型
	 * @author lipenghe
	 * @param  str
	 * @param  format
	 * @param  defDate
	 * @return Date
	 */
	public static Date toDateTime(String str,String format,Date defDate){
		if(str!=null&&str.length()>0){
			try {
				return getSimpleDateFormat(format).parse(str);
			} catch (ParseException e) {
				logger.error("日期字符串不合法",e);
			}
			return defDate;
		}
		return defDate;
	}

	/**
	 * 带异常处理的Object转换长日期:yyyy-MM-dd HH:mm:ss,无法转换返回null
	 * @author wangxuetao
	 * @param obj
	 * @return Date
	 */
	public static Date toDateTime(Object obj,Date defDate){
		if(obj == null){
			return defDate;
		}else{
			if(obj instanceof Date){
				return (Date)obj;
			}else if(obj instanceof String){
				String str=obj.toString();
				if(str.length()>0){
					try {
						return getSimpleDateFormat(yyyy_MM_dd_HH_mm_ss).parse(obj.toString());
					} catch (ParseException e) {
						e.printStackTrace();
					}
				}
			}
			return defDate;
		}
	}

	/**
	 * 标准日期Object yyyy-MM-dd HH:mm:ss 截短
	 * @author wangxuetao
	 * @param obj
	 * @return String
	 */
	public static String subDateTimeStr(Object obj){
		if(obj !=null){
			String str=null;
			if(obj instanceof String){
				str=obj.toString().trim();
				if(str!=null&&str.length()>0){
					if(str.length()>=19){
						return str.substring(0, 19);
					}else if(str.length()>=10){
						return str.substring(0, 10);
					}else{
						return str;
					}
				}
			}else if(obj instanceof Date){
				str=getSimpleDateFormat(yyyy_MM_dd_HH_mm_ss).format((Date)obj);
				return str.substring(0, 19);
			}
			return  "";
		}
		return  "";
	}

	/**
	 * 标准日期string yyyy-MM-dd HH:mm:ss 截短
	 * @author wangxuetao
	 * @param str
	 * @return String
	 */
	public static String subDateTimeStr(String str){
		if(str!=null){
			if(str.length()>=19){
				return str.substring(0, 19);
			}else if(str.length()>=10){
				return str.substring(0, 10);
			}else{
				return str;
			}
		}else{
			return  "";
		}
	}

	/**
	 * 不带异常处理的可自定义转换格式的日期类型转换字符串类型
	 * @author lipenghe
	 * @param date
	 * @param format
	 * @return String
	 */
	public static String toDateTimeStr(Date date,String format){
		if(date!=null){
			return getSimpleDateFormat(format).format(date);
		}else{
			return "";
		}
	}

	/**
	 *  不带异常处理的日期类型转换字符串类型
	 * @author lipenghe
	 * @param date
	 * @return String
	 */
	public static String toDateStr(Date date){
		if(date!=null){
			return getSimpleDateFormat(yyyy_MM_dd).format(date);
		}else{
			return "";
		}
	}

	/**
	 *  不带异常处理的日期类型转换字符串类型
	 * @author lipenghe
	 * @param date
	 * @return String
	 */
	public static String toDateTimeStr(Date date){
		if(date!=null){
			return getSimpleDateFormat(yyyy_MM_dd_HH_mm_ss).format(date);
		}else{
			return "";
		}
	}


	/**
	 * 获取当前系统时间 yyyy-MM-dd HH:mm:ss格式
	 * @author wangxuetao
	 * @return String
	 */
	public static String getSysDateTimeStr(){
		return getSimpleDateFormat(yyyy_MM_dd_HH_mm_ss).format(new Date());

	}

	/**
	 * 获取当前系统日期  ,自定义格式
	 * @author wangxuetao
	 * @param  format
	 * @return String
	 */
	public static String getSysDateStr(String format){
		SimpleDateFormat sdf=getSimpleDateFormat(format);
		return sdf.format(new Date());
	}

	/**
	 * 获取当前系统日期  yyyy-MM-dd 格式
	 * @author wangxuetao
	 * @return String
	 */
	public static String getSysDateStr(){
		return getSimpleDateFormat(yyyy_MM_dd).format(new Date());
	}

	/**
	 * 不带异常处理的可自定义转换格式的获取当前日期字符串
	 * @author lipenghe
	 * @param  format
	 * @return String str
	 */
	public static String getSysDateTimeStr(String format){
		return getSimpleDateFormat(format).format(new Date());
	}

	/**
	 * 当前日期截止到日(日期格式)
	 * @author lipenghe
	 * @return Date
	 */
	public static Date getSysDate(){
		Date date = new Date();
		String dateStr = getSimpleDateFormat(yyyy_MM_dd).format(date);
		try {
			date=getSimpleDateFormat(yyyy_MM_dd).parse(dateStr);
		} catch (ParseException e) {
			logger.error("日期字符串不合法",e);
		}
		return date;
	}

	/**
	 * 当前月第一天
	 * @author lipenghe
	 * @return String
	 */
	public static Date getMonthFirstDate(){
		Calendar calendar =null;
		calendar=Calendar.getInstance();//日历对象
		//calendar.setTime(new Date());//设置当前日期
		//calendar.set(Calendar.YEAR, Calendar.MONTH, Calendar.DATE);
		calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		//输出格式化的日期
		return calendar.getTime();
	}

	/**
	 * 指定月第一天
	 * @param yearDiff
	 * @param monthDiff
	 * @return
	 */
	public static Date getMonthFirstDate(int yearDiff,int monthDiff){
		Calendar calendar =null;
		calendar=Calendar.getInstance();//日历对象
		//calendar.setTime(new Date());//设置当前日期
		//calendar.set(Calendar.YEAR, Calendar.MONTH, Calendar.DATE);
		calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
		calendar.add(Calendar.YEAR, yearDiff);
		calendar.add(Calendar.MONTH, monthDiff);
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		//输出格式化的日期
		return calendar.getTime();
	}

	/**
	 * 当前月最后一天
	 * @author lipenghe
	 * @return String
	 */
	public static Date getMonthLastDate(){
		Calendar calendar =null;
		calendar=Calendar.getInstance();//日历对象
		//calendar.setTime(new Date());//设置当前日期
		//calendar.set(Calendar.YEAR, Calendar.MONTH, Calendar.DATE);
		calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
		calendar.set(Calendar.HOUR_OF_DAY, 23);
		calendar.set(Calendar.MINUTE, 59);
		calendar.set(Calendar.SECOND, 59);
		//输出格式化的日期
		return calendar.getTime();
	}

	/**
	 * 指定月最后一天
	 * @param yearDiff
	 * @param monthDiff
	 * @return
	 */
	public static Date getMonthLastDate(int yearDiff,int monthDiff){
		Calendar calendar =null;
		calendar=Calendar.getInstance();//日历对象
		//calendar.setTime(new Date());//设置当前日期
		//calendar.set(Calendar.YEAR, Calendar.MONTH, Calendar.DATE);
		calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
		calendar.add(Calendar.YEAR, yearDiff);
		calendar.add(Calendar.MONTH, monthDiff);
		calendar.set(Calendar.HOUR_OF_DAY, 23);
		calendar.set(Calendar.MINUTE, 59);
		calendar.set(Calendar.SECOND, 59);
		//输出格式化的日期
		return calendar.getTime();
	}
	public synchronized static String FormatDateCnD(String dateTime) {
		StringBuffer res = new StringBuffer();
		if (dateTime == null || dateTime.length() < 8) {
			return dateTime;
		}
		res.append(dateTime.substring(0, 4));
		res.append("年");
		res.append(dateTime.substring(4, 6));
		res.append("月");
		res.append(dateTime.substring(6, 8));
		res.append("日");

		return res.toString();
	}
	/**
	 * 日期加 时/分/秒
	 * @param date
	 * @param hourDiff
	 * @param minuteDiff
	 * @param secondDiff
	 * @param fromNow
	 * @return
	 */
	public static Date addTime(Date date,int hourDiff,int minuteDiff,int secondDiff,boolean fromNow){
		Calendar calendar =null;
		calendar=Calendar.getInstance();//日历对象
		calendar.setTime(date);//设置当前日期
		if(!fromNow){
			calendar.set(Calendar.HOUR_OF_DAY, 0);//时
			calendar.set(Calendar.MINUTE, 0);//分
			calendar.set(Calendar.SECOND, 0);//秒
		}
		calendar.add(Calendar.HOUR_OF_DAY, hourDiff);//时
		calendar.add(Calendar.MINUTE, minuteDiff);//分
		calendar.add(Calendar.SECOND, secondDiff);//秒
		date=calendar.getTime();//输出格式化的日期
		return date;
	}
	
	public static List<String> sysDate(){
		
		List<String> lst = new ArrayList<>();
		
	    Date now = new Date();
	    
	    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");//可以方便地修改日期格式
	    
	    // Date now = dateFormat.parse("2020-04-05");
	    
	    String MTime = dateFormat.format( now );
	    Date date1=null;
	    try {
	        date1 = dateFormat.parse(MTime);//系统当前时间
	    } catch (Exception e) {
	        e.printStackTrace();
	    }
	    Calendar cal = Calendar.getInstance();
	    cal.setTime(date1);
	    
	    int hour=cal.get(Calendar.DAY_OF_WEEK);
	    
	    System.out.println("当前系统日期"+MTime);
	    String time1=null;
	    String time2=null;
	    String time3=null;
	    String time4=null;        
	    String time5=null; 
	    String time6=null; 
	    String time7=null;
	    
	    if(hour == 2) {
	    	time1 = MTime;
	    	System.out.println("1" + time1);
	    }else if(hour == 3) {
	    	cal.add(Calendar.DAY_OF_MONTH, -1);//获取当前日期的前一天.
	        String  date12=dateFormat.format(cal.getTime());
	        time1=date12;
	        System.out.println("1" + time1);
	    }else if(hour == 4) {
	    	cal.add(Calendar.DAY_OF_MONTH, -2);//获取当前日期的前一天.
	        String  date12=dateFormat.format(cal.getTime());
	        time1=date12;
	        System.out.println("1" + time1);
	    }else if(hour == 5) {
	    	// 周一
		    cal.add(Calendar.DAY_OF_MONTH, -3);//获取取当前日期的前三天.
	        String  date14 = dateFormat.format(cal.getTime());
	        time1=date14;
	        System.out.println("1" + time1);
	    }else if(hour == 6) {
	    	cal.add(Calendar.DAY_OF_MONTH, -4);//获取当前日期的前一天.
	        String  date12=dateFormat.format(cal.getTime());
	        time1=date12;
	        System.out.println("1" + time1);
	    }else if(hour == 7) {
	    	cal.add(Calendar.DAY_OF_MONTH, -5);//获取当前日期的前一天.
	        String  date12=dateFormat.format(cal.getTime());
	        time1=date12;
	        System.out.println("1" + time1);
	    }else {
	    	cal.add(Calendar.DAY_OF_MONTH, -6);//获取当前日期的前一天.
	        String  date12=dateFormat.format(cal.getTime());
	        time1=date12;
	        System.out.println("1" + time1);
	    }
	    
        // 周二
        cal.add(Calendar.DAY_OF_MONTH, +1);//获取取当前日期的前三天.
        String  date24 = dateFormat.format(cal.getTime());
        time2=date24;
        System.out.println("2" + time2);
        
        // 周三
        cal.add(Calendar.DAY_OF_MONTH, +1);//获取取当前日期的前三天.
        String  date34 = dateFormat.format(cal.getTime());
        time3=date34;
        System.out.println("3" + time3);
        
        // 周四 
        cal.add(Calendar.DAY_OF_MONTH, +1);//获取取当前日期的前三天.
        String  date44 = dateFormat.format(cal.getTime());
        time4=date44;
        System.out.println("4" + time4);
        
        // 周五
        cal.add(Calendar.DAY_OF_MONTH, +1);//获取取当前日期的前三天.
        String  date54 = dateFormat.format(cal.getTime());
        time5=date54;
        System.out.println("5" + time5);
        
        // 周六
        cal.add(Calendar.DAY_OF_MONTH, +1);//获取取当前日期的前三天.
        String  date64 = dateFormat.format(cal.getTime());
        time6=date64;
        System.out.println("6" + time6);
        
        // 周六
        cal.add(Calendar.DAY_OF_MONTH, +1);//获取取当前日期的前三天.
        String  date74 = dateFormat.format(cal.getTime());
        time7=date74;
        System.out.println("7" + time7);
        
        lst.add(time1);
        lst.add(time2);
        lst.add(time3);
        lst.add(time4);
        lst.add(time5);
        lst.add(time6);
        lst.add(time7);
		
		return lst;
	}
		//获取一个月前的那天
	public static Date getOneMonth() {
		Date now = new Date();
		Date startDate = DateUtils.addDays(now, -30);
		return startDate;
	}
}
