package com.jusfoun.ydylsz.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

/**
 * Created by admin on 2018/7/27.
 */
public class PathTool {
	private static Logger logger= LoggerFactory.getLogger(PathTool.class);
	public static final String FLAG_NORMAL = "N0";
	public static final String FLAG_PROTECTED = "P0";
	public static final String FLAG_UPLOAD = "U0";
	public static final String FLAG_SYSTEMTEMP = "ST";
	public static final String FLAG_USERTEMP = "UT";
	public static final String FLAG_TEMPLATE = "TM";
	public static final String FLAG_LOCALPUB = "LP";
	public static final String FLAG_LOCALPREVIEW = "LV";
	public static final String FLAG_WEBFILE = "W0";
	public static final String FLAG_DOCUMENTSOURCE = "DS";
	public static final String FLAG_SITEFROM = "SF";
	public static final String FLAG_BIGTABLE = "BT";
	public static final String FLAG_INFOVIEW = "IV";
	public static final String FLAG_DOWNLOAD = "D0";
	protected static Map<String,String> MAP=new HashMap<String,String>();
	static{
		MAP.put("N0","norm");
		MAP.put("D0","download");
		MAP.put("P0","protect");
		MAP.put("U0","upload");
		MAP.put("ST","systemp");
		MAP.put("UT","usertemp");
		MAP.put("TM","template");
		MAP.put("LP","pub");
		MAP.put("LV","preview");
		MAP.put("W0","webpic");
		MAP.put("DS","documentsrc");
		MAP.put("SF","sitefrom");
		MAP.put("IV","InfoView");
	}

	/**
	 * 获取当前项目的根地址,如http://localhost:8080/project
	 * @author lipenghe
	 * @param request
	 * @return String
	 */
	public static String getFullServerUrl(HttpServletRequest request){
		/*StringBuffer sb=null;
		sb=new StringBuffer();
		sb.append(request.getScheme());
		sb.append("://");
		sb.append(request.getServerName());
		sb.append(":");
		sb.append(request.getServerPort());*/
		StringBuffer sb=request.getRequestURL();
		sb.delete(sb.length()-request.getRequestURI().length(),sb.length());
		sb.append(request.getContextPath());
		return sb.toString();
	}

	/**
	 * 获取当前项目的项目名前的根地址,如http://localhost:8080
	 * @author li.penghe
	 * @param request
	 * @return String
	 */
	public static String getFullHostUrl(HttpServletRequest request,String ...s){
		/*StringBuffer sb=null;
		sb=new StringBuffer();
		sb.append(request.getScheme());
		sb.append("://");
		sb.append(request.getServerName());
		sb.append(":");
		sb.append(request.getServerPort());*/
		StringBuffer sb=request.getRequestURL();
		sb.delete(sb.length()-request.getRequestURI().length(),sb.length());
		String temp;
		if(s!=null){
			sb.append("/");
			for(int i=0,len=s.length;i<len;i++){
				temp=s[i];
				if(temp!=null&&temp.length()>0){
					if(s[i].startsWith("/")){
						sb.append(s[i].substring(1));
					}else{
						sb.append(s[i]);
					}
					sb.append("/");
				}
			}
		}
		return sb.toString();
	}

	/**
	 * 获取图片服务器的根地址,如http://localhost:8080/webpic
	 * @author li.penghe
	 * @param request
	 * @return String
	 */
	public static String getWebpicPrefix(HttpServletRequest request,String s){
		if(s==null){
			s=request.getContextPath();
		}
		return getFullHostUrl(request,"webpic",s);
	}

	/**
	 * 获取附件服务器的根地址,如http://localhost:8080/upload(附件一般不允许直接访问,所以不推荐使用)
	 * @author li.penghe
	 * @param request
	 * @return String
	 */
	public static String getUploadPrefix(HttpServletRequest request,String s){
		if(s==null){
			s=request.getContextPath();
		}
		return getFullHostUrl(request,"upload",s);
	}

	/**
	 * 获取下载服务器的根地址,如http://localhost:8080/webpic
	 * @author li.penghe
	 * @param request
	 * @return String
	 */
	public static String getDownloadPrefix(HttpServletRequest request,String s){
		if(s==null){
			s=request.getContextPath();
		}
		return getFullHostUrl(request,"download",s);
	}

	/**
	 * 根据原始文件名,日期生成 压缩版本的 新文件名
	 * @param sysFileName
	 * @param date
	 * @return
	 */
	public  static String makeScaleFileName(String sysFileName,Date date){
		String scale="scale";
		String name=FileTool.getName(sysFileName);
		String ext=FileTool.getFileExt(sysFileName);
		if(name.endsWith(scale)){
			return sysFileName;
		}else{
			return name+scale+"."+ext;
		}
	}

	/**
	 * 根据系统文件名,日期 生成压缩的文件名
	 * @param sysFileName
	 * @param date
	 * @return
	 */
	public  static String readFileNameByScaleName(String sysFileName,Date date){
		String scale="scale";
		String name=FileTool.getName(sysFileName);
		if(name.endsWith(scale)){
			return sysFileName.replace(scale,"");
		}else{
			return sysFileName;
		}
	}

	/**
	 * 根据原始文件名,日期生成附图文件名
	 * @param origFileName
	 * @param date
	 * @return
	 */
	public  static String makeWebpicName(String origFileName,Date date){
		if(origFileName!=null&&origFileName.length()>0){
			return makeNextSysFileName("W0",origFileName,date);
		}
		return null;
	}

	/**
	 * 根据原始文件名,日期生成附件文件名
	 * @param origFileName
	 * @param date
	 * @return
	 */
	public  static String makeUploadName(String origFileName,Date date){
		if(origFileName!=null&&origFileName.length()>0){
			return makeNextSysFileName("U0",origFileName,date);
		}
		return null;
	}

	/**
	 * 根据原始文件名,日期生成下载文件名
	 * @param origFileName
	 * @param date
	 * @return
	 */
	public  static String makeDownloadName(String origFileName,Date date){
		if(origFileName!=null&&origFileName.length()>0){
			return makeNextSysFileName("D0",origFileName,date);
		}
		return null;
	}

	/**
	 * 根据路径标示,原始文件名,日期生成新版文件名
	 * @param sign
	 * @param origFileName
	 * @param date
	 * @return
	 */
	private  static String makeNextSysFileName(String sign,String origFileName,Date date){
		if(date==null){
			date=new Date();
		}
		String ext= FileTool.getFileExt(origFileName);
		StringBuffer sb=null;
		sign=sign.toUpperCase();
		if(MAP.get(sign)!=null){
			sb=new StringBuffer(sign);
			sb.append(DateTool.toDateTimeStr(date,"yyyyMMddHHmmssSSS"));
			sb.append(String.format("%06d",new Random().nextInt(100000)));
			sb.append(".");
			sb.append(ext.toLowerCase());
			return  sb.toString();
		}else{
			logger.error("错误的路径标示{}",sign);
			return null;
		}
	}

	/**
	 * 传入系统转存后的文件名,获取新版文件标示
	 * @param sysFileName
	 * @return
	 */
	public static String getFileFlag(String sysFileName){
		String flag=null;
		if(sysFileName!=null&&sysFileName.length()>2){
			flag=sysFileName.substring(0,2).toUpperCase();
			if(MAP.get(flag)!=null){
				return flag;
			}else{
				logger.error("文件名不符合规则");
				return null;
			}
		}else{
			logger.error("文件名不符合规则");
			return null;
		}
	}

	/**
	 * 根据新版文件名获取文件的网络路径
	 * @param request
	 * @param sysFileName
	 * @return
	 */
	public static String readWebPath(HttpServletRequest request,String sysFileName){
		return getFilePath(getFullHostUrl(request,null),request.getContextPath(),sysFileName);
	}

	/**
	 * 根据新版文件名获取文件的网络路径的父级
	 * @param request
	 * @param sysFileName
	 * @return
	 */
	public static String readParentWebPath(HttpServletRequest request,String sysFileName){
		return getFileParentPath(getFullHostUrl(request,null),request.getContextPath(),sysFileName);
	}

	/**
	 * 根据新版文件名获取压缩文件的网络路径
	 * @param request
	 * @param sysFileName
	 * @return
	 */
	public static String readScaleWebPath(HttpServletRequest request,String sysFileName){
		return getScaleFilePath(getFullHostUrl(request,null),request.getContextPath(),sysFileName);
	}

	/**
	 * 根据新版文件名获取文件的网络路径
	 * @param request
	 * @param sysFileName
	 * @return
	 */
	public static String readAbsolutePath(HttpServletRequest request,String sysFileName){
		return getFilePath(null,request.getContextPath(),sysFileName);
	}
	
	/**
	 * 根据新版文件名获取文件的网络路径
	 * @param request
	 * @param sysFileName
	 * @return
	 */
	public static String readLicAbsolutePath(HttpServletRequest request,String code,String sysFileName){
		return getFilePath(null,code+request.getContextPath(),sysFileName);
	}

	/**
	 * 根据新版文件名获取文件的网络路径
	 * @param request
	 * @param sysFileName
	 * @return
	 */
	public static String readScaleAbsolutePath(HttpServletRequest request,String sysFileName){
		return getScaleFilePath(null,request.getContextPath(),sysFileName);
	}

	/**
	 * 根据新版文件名获取文件的网络路径
	 * @param prefix
	 * @param sysFileName
	 * @return
	 */
	public static String getFileParentPath(String prefix,String context,String sysFileName){
		//W020180728104839548094083.png
		String flag=getFileFlag(sysFileName);
		int len=0;
		len=FileTool.getName(sysFileName).length();
		if(flag!=null&& len==25||len==30){
			flag=flag.toUpperCase();
			String sign=MAP.get(flag);
			if(context!=null&&context.length()>0){
				context=context.replace("/","");
			}
			return FileTool.concatFilePath(prefix,null,
					sign,context,sysFileName.substring(2,8),sysFileName.substring(2,10));
		}else{
			logger.error("文件不符合系统文件名规则");
			return null;
		}
	}

	/**
	 * 根据新版文件名获取文件的网络路径
	 * @param prefix
	 * @param sysFileName
	 * @return
	 */
	public static String getFilePath(String prefix,String context,String sysFileName){
		//W020180728104839548094083.png
		String flag=getFileFlag(sysFileName);
		int len=0;
		if(flag!=null&& (len=FileTool.getName(sysFileName).length())==25){
			flag=flag.toUpperCase();
			String sign=MAP.get(flag);
			if(context!=null&&context.length()>0){
				context=context.replaceAll("^/","");
			}
			return FileTool.concatFilePath(prefix,sysFileName,
					sign,context,sysFileName.substring(2,8),sysFileName.substring(2,10));
		}else{
			logger.error("文件不符合系统文件名规则");
			return null;
		}
	}

	/**
	 * 根据新版文件名获取压缩文件的网络路径
	 * @param prefix
	 * @param sysFileName
	 * @return
	 */
	public static String getScaleFilePath(String prefix,String context,String sysFileName){
		//W020180728104839548094083scale.png
		String flag=getFileFlag(sysFileName);
		int len=0;
		if(flag!=null&& (len=FileTool.getName(sysFileName).length())==30){
			flag=flag.toUpperCase();
			String sign=MAP.get(flag);
			if(context!=null&&context.length()>0){
				context=context.replace("/","");
			}
			return FileTool.concatFilePath(prefix,sysFileName,
					sign,context,sysFileName.substring(2,8),sysFileName.substring(2,10));
		}else{
			logger.error("文件不符合系统文件名规则");
			return null;
		}
	}

	/**
	 * 根据新版文件名获取文件的网络路径
	 * @param prefix
	 * @param fileName
	 * @param date
	 * @return
	 */
	public static String getFilePath(String prefix,String sign,String context,String fileName,Date date){
		//W020180728104839548094083.png
		if(date==null){
			date=new Date();
		}
		if(context!=null&&context.length()>0){
			context=context.replace("/","");
		}
		return FileTool.concatFilePath(prefix,fileName,sign,context,
				DateTool.toDateTimeStr(date,"yyyyMM"),DateTool.toDateTimeStr(date,"yyyyMMdd"));
	}

	/**
	 * 获取当前项目的物理路径
	 * @author lipenghe
	 * @param filename
	 * @param s
	 * @return String
	 */
	public static String getRealServerPath(String filename,String ...s){
		String projectPath = Thread.currentThread().getContextClassLoader().getResource("").toString();
		projectPath= projectPath.replace("file:/","").replace("/WEB-INF/classes/","");
		System.out.println("======================1"+projectPath);
		projectPath = projectPath.split("webapps")[0]+"webapps";

		System.out.println("======================22"+projectPath);
		if(File.separator.equals("/")){
			projectPath="/"+projectPath;
		}
		StringBuffer sb=new StringBuffer();
		sb.append(projectPath);
		if(s!=null){
			for(String temp:s){
				if(temp!=null&&temp.length()>0){
					sb.append(temp).append("/");
				}
			}
		}
		if(filename!=null){
			sb.append(filename);
		}
		return sb.toString();
	}

	/**
	 * 获取系统文件的物理路径
	 * @author lipenghe
	 * @param request
	 * @param sysFileName
	 * @return String
	 */
	public static String readRealFilePath(HttpServletRequest request, String sysFileName){
		String context=request.getContextPath();
		String path=getRealServerPath(null,null);
		int index=path.lastIndexOf(context);
		if(index>0){
			path=path.substring(0,index);
		}
		return getFilePath(path,request.getContextPath(),sysFileName);
	}

	/**
	 * 获取当前项目的物理路径
	 * @author lipenghe
	 * @param prefix
	 * @param fileName
	 * @param date
	 * @return String
	 */
	public static String getRealFilePath(String prefix, String sign,String context,String fileName,Date date){
		String directory= getFilePath(prefix,sign,context,null,date);
		File file=new File(directory);
		if(!file.exists()){
			file.mkdirs();
		}
		if(fileName==null||fileName.length()==0){
			fileName="";
		}
		return directory+fileName;
	}

	/**
	 * 获取图片的物理路径
	 * @author lipenghe
	 * @param request
	 * @param fileName
	 * @param date
	 * @return String
	 */
	public static String makeRealWebpicPath(HttpServletRequest request,String fileName,Date date){
		String context=request.getContextPath();
		String path=getRealServerPath(null,null);
		int index=path.lastIndexOf(context);
		if(index>0){
			path=path.substring(0,index);
		}
		if(fileName==null||fileName.length()==0){
			return getRealFilePath(path,"webpic",request.getContextPath(),null,date);
		}else{
			String filePath=getRealFilePath(path,"webpic",request.getContextPath(),makeUploadName(fileName,date),date);
			File file=new File(filePath);
			while(file.exists()){
				filePath= makeRealWebpicPath(request,fileName,date);
			}
			return filePath;
		}
	}
	/**
	 * 获取电子证照图片的物理路径
	 * @author lipenghe
	 * @param request
	 * @param fileName
	 * @param date
	 * @return String
	 */
	public static String makeLicsRealWebpicPath(HttpServletRequest request,String fileName,Date date, String code){
		String context=request.getContextPath();
		String path=getRealServerPath(null,null);
		int index=path.lastIndexOf(context);
		if(index>0){
			path=path.substring(0,index);
		}
		if(fileName==null||fileName.length()==0){
			return getRealFilePath(path,"webpic"+"/"+code,request.getContextPath(),null,date);
		}else{
			String filePath=getRealFilePath(path,"webpic"+"/"+code,request.getContextPath(),makeUploadName(fileName,date),date);
			File file=new File(filePath);
			while(file.exists()){
				filePath= makeRealWebpicPath(request,fileName,date);
			}
			return filePath;
		}
	}

	/**
	 * 获取文件的物理路径
	 * @author lipenghe
	 * @param request
	 * @param fileName
	 * @param date
	 * @return String
	 */
	public static String makeRealUploadPath(HttpServletRequest request,String fileName,Date date){
		String context=request.getContextPath();
		String path=getRealServerPath(null,null);
		int index=path.lastIndexOf(context);
		if(index>0){
			path=path.substring(0,index);
		}
		if(fileName==null||fileName.length()==0){
			return getRealFilePath(path,"upload",request.getContextPath(),null,date);
		}else{
			String filePath=getRealFilePath(path,"upload",request.getContextPath(),makeUploadName(fileName,date),date);
			File file=new File(filePath);
			while(file.exists()){
				filePath= makeRealUploadPath(request,fileName,date);
			}
			return filePath;
		}
	}

	/**
	 * 获取图片的物理路径
	 * @author lipenghe
	 * @param request
	 * @param fileName
	 * @param date
	 * @return String
	 */
	public static String makeRealDownloadPath(HttpServletRequest request,String fileName,Date date){
		String context=request.getContextPath();
		String path=getRealServerPath(null,null);
		int index=path.lastIndexOf(context);
		if(index>0){
			path=path.substring(0,index);
		}
		if(fileName==null||fileName.length()==0){
			return getRealFilePath(path,"download",request.getContextPath(),null,date);
		}else{
			String filePath=getRealFilePath(path,"download",request.getContextPath(),makeDownloadName(fileName,date),date);
			File file=new File(filePath);
			while(file.exists()){
				filePath= makeRealDownloadPath(request,fileName,date);
			}
			return filePath;
		}
	}

	public static void main(String[] args) {
		System.out.println(makeWebpicName("是的.png",null));
		System.out.println(getFilePath("","hlwbgpt","W020180728104839548094083.png"));
		System.out.println(getRealServerPath("W020180728104839548094083.png",""));
		System.out.println(getRealFilePath("","webpic","hlwbgpt","是的.png",null));
	}
}
