package com.jusfoun.ydylsz.util;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

import javax.annotation.PostConstruct;
import java.util.List;

@Component
@Slf4j
public class RedisUtil {

    //服务器IP地址
//    private static String ADDR = "10.223.3.64";
    private static String ADDR = "172.16.0.73";
    // private static String ADDR = "172.16.0.2";

    //端口
    private static int PORT = 6379;

    //密码
    private static String AUTH = "redis@123";
    //连接实例的最大连接数
    private static int MAX_ACTIVE = 1024;

    //控制一个pool最多有多少个状态为idle(空闲的)的jedis实例，默认值也是8。
    private static int MAX_IDLE = 200;

    //等待可用连接的最大时间，单位毫秒，默认值为-1，表示永不超时。如果超过等待时间，则直接抛出JedisConnectionException
    private static int MAX_WAIT = 10000;

    //连接超时的时间
    private static int TIMEOUT = 10000;

    // 在borrow一个jedis实例时，是否提前进行validate操作；如果为true，则得到的jedis实例均是可用的；
    private static boolean TEST_ON_BORROW = true;

    private static JedisPool jedisPool = null;

    @Value(value = "${spring.redis.host}")
    private String configHost;


    /**
     * 初始化Redis连接池
     */
    @PostConstruct
    void init() {
        try {
            ADDR = configHost;
            log.info("redis 地址  {}", ADDR);
            JedisPoolConfig config = new JedisPoolConfig();
            config.setMaxTotal(MAX_ACTIVE);
            config.setMaxIdle(MAX_IDLE);
            config.setMaxWaitMillis(MAX_WAIT);
            config.setTestOnBorrow(TEST_ON_BORROW);
            jedisPool = new JedisPool(config, ADDR, PORT, TIMEOUT, AUTH);
            log.info("--------- jedis init success -----------------------------");
        } catch (Exception e) {

            e.printStackTrace();
        }
    }

    /**
     * 获取Jedis实例
     */
    public synchronized static Jedis getJedis() {
        try {
            if (jedisPool != null) {
                Jedis resource = jedisPool.getResource();
                return resource;
            } else {
                log.error("jedisPoll 错误 变成null了！！！！！！！！！！！！！！");
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     *
     * @param dbIndex redis db-index
     * @return
     */
    public synchronized static Jedis getJedis(int dbIndex) {
        try {
            if (jedisPool != null) {
                Jedis resource = jedisPool.getResource();
                resource.select(dbIndex);
                return resource;
            } else {
                log.error("jedisPoll 错误 变成null了！！！！！！！！！！！！！！");
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /***
     *
     * 释放资源
     */
    public static void returnResource(final Jedis jedis) {
        if(jedis != null) {
            jedisPool.returnResource(jedis);
        }
    }

    public static void changeDataBase(int dbIndex){
        jedisPool.getResource().select(dbIndex);
        log.info("change database {}", dbIndex);
    }

}
